package no.stud;

import backtype.storm.Config;
import backtype.storm.Constants;
import backtype.storm.LocalCluster;
import backtype.storm.StormSubmitter;
import backtype.storm.topology.TopologyBuilder;
import backtype.storm.tuple.Fields;
import backtype.storm.generated.*;

import no.stud.bolts.FilterForwardSkylineBolt;
import no.stud.bolts.IncrForwardSkylineBolt;
import no.stud.bolts.SimpleSkylineBolt;
import no.stud.spouts.BinFileReadSpout;
import no.stud.spouts.GenerateDatasetSpout;
import no.stud.util.CustomConfig;
import no.stud.util.DistributionType;


public class FilteredImprovedSkylineTopology {
	public static void main(String[] args) throws InterruptedException {
         
        // Topology definition
		TopologyBuilder builder = new TopologyBuilder();
		String topologyName = "Filtered-Improved-Skyline-Topology";

		// ReadDataset from file
 		builder.setSpout("Generator",new BinFileReadSpout(), 2);
 		
 		// FilterForwardBolt
		builder.setBolt("Filtered", new FilterForwardSkylineBolt() , 2).setNumTasks(4).localOrShuffleGrouping("Generator");
		
		//  
		builder.setBolt("final", new IncrForwardSkylineBolt() , 2).localOrShuffleGrouping("Filtered");
		
		// SimpleSkylineBolt
		builder.setBolt("skyline", new SimpleSkylineBolt() , 1).localOrShuffleGrouping("final");
		
		
		
        // Configuration
		Config conf = new Config();			// Creat a new config, to submit with the topology.
 		// conf.put(submitArgs, args[0]);			// Put arguments from ./storm jar file.jar Topology args1 argsn in configfile.

		/** Data generator configuration. **/
		// Max value for a dimension in the point.
		conf.put(CustomConfig.DimensionMaxValue, 1000000);
		
		// How many dimensions a point has.
		conf.put(CustomConfig.PointDimensions, 5);		
		
		// What type of generator shall be used. /* Uniform, AntiCorrelated, Correlated */
		conf.put(CustomConfig.GeneratorType, DistributionType.ANTICORRELATED.getId());

		// Set the toggle sleep value, to enable to throttle the spouts.
		conf.put(CustomConfig.ToggleSleepValue, 4);
		
		// Set the sleep time in ms, default 1ms when enabled.
		conf.put(CustomConfig.ToggleSleepTime, 5);
		
		/** Topology configuration **/
		// Disabling reliability. (Setting how many executers to spawn for acking.)
		conf.put(Config.TOPOLOGY_ACKER_EXECUTORS, 0);
		
		// Bolts and Spouts will log eg. every tuppel they emit.
		conf.setDebug(false);
		
		// Maximum pending tuples, has no effect when reliability is disabled.
		conf.setMaxSpoutPending(5000);
		
		// Set the max number of storm workers. Remember acker is using one task.
		conf.setNumWorkers(4);		
		
		// Fallback on java is extreme expencive and is there only for prototyping. 
 		conf.put(Config.TOPOLOGY_FALL_BACK_ON_JAVA_SERIALIZATION, false);
 		
 		// Make storm ignore kryo registrations that ain't availble on the classpath.
 		conf.put(Config.TOPOLOGY_SKIP_MISSING_KRYO_REGISTRATIONS, true);
 		
 		// Register kyro serialization for the double[] datatype.
 		conf.registerSerialization(double[].class);
 		
 		
 		/** JVM options **/
 		// Set a very aggresive GBC, as we mostly relay on generating data and droping it, also tuning memory settings eg.
		conf.put(Config.TOPOLOGY_WORKER_CHILDOPTS, "-d64 -XX:GCTimeRatio=1 -XX:+AggressiveOpts -XX:+UseLargePages -XX:MaxNewSize=4g ");
		// internal note.
 		// conf.put(Config.TOPOLOGY_WORKER_CHILDOPTS, "-Xmx768m -XX:MaxPermSize=1024m");
		
		// Deny config from automaticly getting some options two times.
		conf.put(Config.WORKER_CHILDOPTS, " "); 
 		
		/** Nimbus fails heartbeat **/
		conf.put(conf.TOPOLOGY_EXECUTOR_RECEIVE_BUFFER_SIZE,1024);
		conf.put(conf.TOPOLOGY_EXECUTOR_SEND_BUFFER_SIZE,1024);
		conf.put(conf.NIMBUS_TASK_TIMEOUT_SECS,5);
		
		
		/** ZMQ (ZeroMQ), flow control parameters. ZMQ is fire and forget **/
		// Buffersize in items before, it takes action eg. blocking. Infinite 
		// or a abitary high number will make it eat all your memory. If all bolts can't keep up.
		conf.put(Config.ZMQ_HWM, 0); // Controlling the behaivor in spots\bolts instead.
		// Number of ZMQ threads that should be used by the ZMQ context in each worker process.
 		conf.put(Config.ZMQ_THREADS, 1);
 		// Setting the timeout for ZMQ
 		conf.put(Config.ZMQ_LINGER_MILLIS, 5000);	


 		
 		// Decide on to run local or remote dipending on args.
 		if (args != null && args.length == 0) {
     		
 			// Topology run cluster
 			try{	
 				// Submit the topology.
 				StormSubmitter.submitTopology(topologyName, conf, builder.createTopology());
			
 			// Will be catched of storm and printed.
 			} catch (Exception ex) {	
 				ex.printStackTrace();
 				/*	Throws:	
					AlreadyAliveException - if a topology with this name is already running
					InvalidTopologyException - if an invalid topology was submitted
 				 */
 			}
 		} else if(args.length > 0) {
 			
 		// Local topology testing
	
 			// Local cluster config changes
 			conf.setMaxSpoutPending(1);			// Maximum pending tuples, has no effect when reliability is disabled.
 			conf.setDebug(true);				// Bolts and Spouts will log eg. every tuppel they emit.

 			// Topology run local
 			LocalCluster cluster = new LocalCluster();	// Creat a local cluster
 			cluster.submitTopology("LocalToplogie", conf, builder.createTopology());	// Submit the topology to the local cluster.
		
 			// Delay the shutdown of the local-cluster.
 			Thread.sleep(300000);				// Delay in ms. ~5 min.
 			cluster.shutdown();				// Shut down the local cluster.
 		}
	} // end function
} // end class
